// src/components/ItemDetails.js
import React, { useState, useEffect, useRef, useCallback } from 'react';
import { TextField, Button, Container, Avatar, Tooltip } from '@mui/material';
import axios from 'axios';
import { PRIMARY_COLOR, SECONDARY_COLOR } from '../App';
//import { useNavigate } from 'react-router-dom'; // Import useNavigate

export default function ItemDetails({ item, token, onSave, onClose, boxId }) {
  const [name, setName] = useState(item.name);
  const [description, setDescription] = useState(item.description);
  const [imagePath, setImagePath] = useState(item.image_path || '');
  const [imageSrc, setImageSrc] = useState('/images/default.jpg'); // Initial default image
  const fileInputRef = useRef(null); // Add this line to define fileInputRef
  const [imageOverlayVisible, setImageOverlayVisible] = useState(false);
  // const navigate = useNavigate(); // Initialize useNavigate
  // eslint says boxName is defined but never used, but when I remove it it fails to compile
  // because boxName is undefined
  // eslint-disable-next-line
  const [boxName, setBoxName] = useState('');
  const [boxes, setBoxes] = useState([]);
  const [selectedBoxId, setSelectedBoxId] = useState(item.box_id);

  //console.log("item.box_id: " + item.box_id); 

  useEffect(() => {
    const fetchBoxes = async () => {
      try {
        const response = await axios.get(`${process.env.REACT_APP_API_URL}/boxes`, {
          headers: { Authorization: `Bearer ${token}` }
        });
        setBoxes(response.data);
      } catch (error) {
        console.error('Error fetching boxes:', error);
      }
    };
    fetchBoxes();
  }, [token]);


  const handleBoxChange = (event) => {
    const newBoxId = event.target.value;
    setSelectedBoxId(newBoxId); // Update only this state
    //console.log('Selected box ID:', newBoxId);
  };

  useEffect(() => {
    // Fetch box details only when the selectedBoxId changes
    const getBoxDetails = async (boxId) => {
      try {
        const boxIdNumber = +boxId;
        if (isNaN(boxIdNumber)) {
          console.error('Invalid boxId:', boxId);
          return;
        }
        const response = await axios.get(`${process.env.REACT_APP_API_URL}/boxes/${boxIdNumber}`, {
          headers: { Authorization: `Bearer ${token}` }
        });
        setBoxName(response.data.name);
      } catch (error) {
        console.error('Error fetching box details:', error);
      }
    };

    if (selectedBoxId !== item.box_id) {
      getBoxDetails(selectedBoxId);  // Fetch when selected box changes
      //console.log("selectedBoxId:", selectedBoxId);
    } else if (item.box_id) {
      getBoxDetails(item.box_id);  // Fetch when boxId exists and selectedBoxId is empty
      //console.log("item.box_id:", item.box_id);
    }
  }, [selectedBoxId, token, item.box_id]); // Removed `boxId` from dependencies

  useEffect(() => {
    // Function to fetch image similar to getImageSrc in Items.js
    const getImageSrc = (itemId) => {
      return axios.get(`${process.env.REACT_APP_API_URL}/items/${itemId}/image`, {
        headers: { Authorization: `Bearer ${token}` },
        responseType: 'blob'
      })
      .then(response => {
        if (response.status === 200) {
          return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.onload = () => resolve(reader.result);
            reader.onerror = reject;
            reader.readAsDataURL(response.data);
          });
        } else {
          throw new Error('Image fetch failed');
        }
      })
      .catch(() => {
        // Return the data URL of the default image if image fetch fails
        return new Promise((resolve, reject) => {
          const img = new Image();
          img.src = '/default.jpg'; 
          img.onload = () => {
            const canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            const ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            const dataURL = canvas.toDataURL();
            resolve(dataURL);
          };
          img.onerror = reject;
        });
      });
    };

    // Fetch the image when the component mounts or the item changes
    getImageSrc(item.ID).then(dataUrl => setImageSrc(dataUrl));
  }, [item.ID, token]);

  // const handleCloseItemDetails = () => {
  //   onClose(); // Call the onClose prop to close the modal
  //   navigate(`/boxes/${boxId}/items`); // Navigate back to the items list
  // };

  const handleImageUpload = useCallback(async () => {
    const formData = new FormData();
    formData.append('image', fileInputRef.current.files[0]);

    try {
      const response = await axios.post(`${process.env.REACT_APP_API_URL}/items/${item.ID}/upload`, formData, {
        headers: {
          Authorization: `Bearer ${token}`,
          'Content-Type': 'multipart/form-data'
        }
      });
      // Handle successful upload (e.g., show a success message)
      //console.log('Image uploaded successfully!', response.data.imagePath);
      return response.data.imagePath; // Indicate successful upload
    } catch (error) {
      // Handle upload error (e.g., show an error message)
      console.error('Image upload failed:', error);
    }
  }, [item.ID, token]);

  // eslint-disable-next-line
  const updateItemBoxId = async () => {
    try {
      // eslint-disable-next-line
      const response = await axios.put(`${process.env.REACT_APP_API_URL}/items/${item.id}`, {
        box_id: selectedBoxId,
      }, {
        headers: { Authorization: `Bearer ${token}` }
      });
      // Update the item's boxId
      item.box_id = selectedBoxId;
    } catch (error) {
      console.error('Error updating item boxId:', error);
    }
  };

  const handleSave = useCallback( async () => {
    let imagePath;
    // 1. Handle image upload first if a new image is selected
    if (fileInputRef.current.files[0]) {
      // eslint-disable-next-line
      imagePath = await handleImageUpload();
    }
    //console.log("Selected box ID:", selectedBoxId)

    // 2. Update item details (name, description, etc.)
    try {
      await axios.put(`${process.env.REACT_APP_API_URL}/items/${item.ID}`, {
        name,
        description,
        box_id: +selectedBoxId, // Ensure the updated selected box is saved
      }, {
        headers: { Authorization: `Bearer ${token}` }
      });
      onSave(); // Notify parent to refresh items
    } catch (error) {
      // Handle update error
      console.error('Item update failed:', error);
    }
  }, [item.ID, name, description, selectedBoxId, token, onSave, handleImageUpload]);

  const handleImageError = (e) => {
    e.target.src = '/images/default.jpg'; // Fallback to default image on error
  };

  const handleAvatarClick = () => {
    setImageOverlayVisible(true);
  };
  
  const handleCloseOverlay = () => {
    setImageOverlayVisible(false);
  };

  return (
    <Container>
      <h3>Edit Item: {item.name}</h3>

      {/* Display the item image as an avatar */}
      <Tooltip title="Click to enlarge">
        <Avatar
          src={imageSrc}
          alt={name}
          onError={handleImageError}
          sx={{ width: 200, height: 200, marginBottom: '16px' }} // Style the Avatar
          onClick={handleAvatarClick}
        />
      </Tooltip>  
      {imageOverlayVisible && (
        <div className="image-overlay">
          <img src={imageSrc} alt={name} />
          <button className="close-button" onClick={handleCloseOverlay}>
            Close
          </button>
        </div>
      )}
      <TextField
        label="Item Name"
        variant="outlined"
        fullWidth
        margin="normal"
        value={name}
        onChange={(e) => setName(e.target.value)}
      />
      <TextField
        label="Item Description"
        variant="outlined"
        fullWidth
        margin="normal"
        value={description}
        onChange={(e) => setDescription(e.target.value)}
      />
      <TextField 
        label="Item Image Path" 
        variant="outlined" 
        fullWidth 
        margin="normal"
        value={imagePath}
        onChange={(e) => setImagePath(e.target.value)} 
        sx={{ display: 'none' }}
      />
      <input 
        type="file" 
        accept="image/*" 
        ref={fileInputRef} 
        style={{ display: 'none' }} 
        id="editItemImageUpload" // Unique ID
      />
      <select value={selectedBoxId} onChange={handleBoxChange}>
        <option value="">No box</option>
        {boxes.map((box) => (
          // eslint-disable-next-line 
          //console.log('Box at the selection point:', box.ID),
          //console.log("Box name:", box.name),
          <option key={box.ID} value={box.ID}>
            {box.name}
          </option>
        ))}
      </select>     
      <br/>
      <br/>
      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" component="label" htmlFor="editItemImageUpload"> 
        Upload Image
        <input 
          type="file" 
          accept="image/*" 
          id="image-upload" 
          style={{ display: 'none' }} 
          onChange={(e) => {
            // You can handle image preview here if needed
            setImagePath(e.target.files[0].name); 
          }}
        />
      </Button>

      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" color="primary" onClick={handleSave}>
        Save Changes
      </Button>
      <Button sx={{ backgroundColor: PRIMARY_COLOR, borderBottom: '1px solid', borderColor: '#444', color: SECONDARY_COLOR }} variant="contained" color="primary" onClick={onClose}>Close</Button>
    </Container>
  );
}